/***************************************************************************
                          highscore.cpp  -  description
                             -------------------
    begin                : Fri May 19 2000
    copyright            : (C) 2000 by Roman Merzlyakov
    email                : roman@sbrf.barrt.ru
    copyright            : (C) 2000 by Roman Razilov
    email                : Roman.Razilov@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <qlineedit.h>
#include <qlayout.h>
#include <qfont.h>
#include <kmsgbox.h>
#include <kseparator.h>

#include "highscore.h"

HScore::HScore():QObject() {
  readHighscore();
}

HScore::~HScore() {
}

void HScore::slotEndOfGame(int score) {

    // create highscore entry
    HighScore hs;
    hs.score = score;

    // check if we made it into Top10
    bool isHighscore = FALSE;
    if(highscore.size() < HIGHSCORE_MAX)
      isHighscore = TRUE;
    else if(isBetter(hs, highscore[HIGHSCORE_MAX-1]))
      isHighscore = TRUE;

    if(isHighscore) {
      QString name = getPlayerName();
      strncpy(hs.name, (const char *)name, sizeof(hs.name) - 1);
      int rank = insertHighscore(hs);
      showHighscore(rank);
    } else {
      QString s;
      s.sprintf( "Congratulations! You score is %i", score );
//      s.sprintf( locale->translate("Congratulations! You score is %i"), score );

      KMsgBox::message(NULL, "End of game", s.data());
//      KMsgBox::message(this, locale->translate("End of game"), s.data());
    }
}

QString HScore::getPlayerName() {
  QDialog *dlg = new QDialog(NULL, "Hall Of Fame", TRUE);

  QLabel  *l1  = new QLabel("You've made in into the \"Hall Of Fame\".Type in\nyour name so mankind will always remember\nyour cool rating.", dlg);
//  QLabel  *l1  = new QLabel(locale->translate("You've made in into the \"Hall Of Fame\".Type in\nyour name so mankind will always remember\nyour cool rating."), dlg);
  l1->setFixedSize(l1->sizeHint());

  QLabel *l2 = new QLabel("Your name:", dlg);
//  QLabel *l2 = new QLabel(locale->translate("Your name:"), dlg);
  l2->setFixedSize(l2->sizeHint());

  QLineEdit *e = new QLineEdit(dlg);
  e->setText("XXXXXXXXXXXXXXXX");
  e->setMinimumWidth(e->sizeHint().width());
  e->setFixedHeight(e->sizeHint().height());
  e->setText("");
  e->setFocus();

  QPushButton *b = new QPushButton(i18n("OK"), dlg);
  b->setDefault(TRUE);
//  if(style() == MotifStyle)
//    b->setFixedSize(b->sizeHint().width() + 10,
//		    b->sizeHint().height() +10);
//  else
  b->setFixedSize(b->sizeHint());
  connect(b, SIGNAL(released()), dlg, SLOT(accept()));
  connect(e, SIGNAL(returnPressed()),
	  dlg, SLOT(accept()));

  // create layout
  QVBoxLayout *tl = new QVBoxLayout(dlg, 10);
  QHBoxLayout *tl1 = new QHBoxLayout();
  tl->addWidget(l1);
  tl->addSpacing(5);
  tl->addLayout(tl1);
  tl1->addWidget(l2);
  tl1->addWidget(e);
  tl->addSpacing(5);
  tl->addWidget(b);
  tl->activate();
  tl->freeze();

  dlg->exec();

  QString s = e->text();
  delete dlg;

  if(s.length() == 0)
    s = " ";
  return s;
}

bool HScore::isBetter(HighScore &hs, HighScore &than) {
  if(hs.score > than.score) return TRUE;
  else return FALSE;
}

int HScore::insertHighscore(HighScore &hs) {
  int i;

  if(highscore.size() == 0) {
    highscore.resize(1);
    highscore[0] = hs;
    writeHighscore();
    return 0;
  } else {
    HighScore last = highscore[highscore.size() - 1];
    if(isBetter(hs, last) || (highscore.size() < HIGHSCORE_MAX)) {
      if(highscore.size() == HIGHSCORE_MAX)
	highscore[HIGHSCORE_MAX - 1] = hs;
      else {
	highscore.resize(highscore.size()+1);
	highscore[highscore.size() - 1] = hs;
      }

      // sort in new entry
      int bestsofar = highscore.size() - 1;
      for(i = highscore.size() - 1; i > 0; i--)
	if(isBetter(highscore[i], highscore[i-1])) {
	  // swap entries
	  HighScore temp = highscore[i-1];
	  highscore[i-1] = highscore[i];
	  highscore[i] = temp;
	  bestsofar = i - 1;
	}

      writeHighscore();
      return bestsofar;
    }
  }
  return -1;
}


void HScore::readHighscore() {
  int i;
  QString s, e, grp;
  KConfig *conf = kapp->getConfig();

  highscore.resize(0);
  i = 0;
  bool eol = FALSE;
  grp = conf->group();
  conf->setGroup("Highscore of Lines");
  while ((i < (int)HIGHSCORE_MAX) && !eol) {
    s.sprintf("Highscore_%d", i);
    if(conf->hasKey(s)) {
      e = conf->readEntry(s.data());
      highscore.resize(i+1);

      HighScore hs;
      memset(hs.name, 0, sizeof(hs.name));
      sscanf((const char *)e, "%d %30c",
	     &hs.score, (char*)&hs.name);
      highscore[i] = hs;
    } else
      eol = TRUE;
    i++;
  }

  // restore old group
  conf->setGroup(grp.data());
}


void HScore::writeHighscore() {
  int i;
  QString s, e, grp;
  KConfig *conf = kapp->getConfig();

  grp = conf->group();
  conf->setGroup("Highscore of Lines");
  for(i = 0; i < (int)highscore.size(); i++) {
    s.sprintf("Highscore_%d", i);
    HighScore hs = highscore[i];
    e.sprintf("%d %30s",
	      hs.score, hs.name);
    conf->writeEntry(s, e);
  }

  // restore old group
  conf->setGroup(grp.data());
}

void HScore::showHighscore(int focusitem)  {
  // this may look a little bit confusing...
  QDialog *dlg = new QDialog(0, "High score", TRUE);
//  QDialog *dlg = new QDialog(0, locale->translate("Hall Of Fame"), TRUE);

  dlg->setCaption("Lines: high score");
//  dlg->setCaption(locale->translate("Shisen-Sho: Hall Of Fame"));

  QVBoxLayout *tl = new QVBoxLayout(dlg, 10);
  
  QLabel *l = new QLabel("High score", dlg);
//  QLabel *l = new QLabel(locale->translate("Hall Of Fame"), dlg);

  QFont f = dlg->font();
//  QFont f = font();

  f.setPointSize(24);
  f.setBold(TRUE);
  l->setFont(f);
  l->setFixedSize(l->sizeHint());
  l->setFixedWidth(l->width() + 32);
  l->setAlignment(AlignCenter);
  tl->addWidget(l);

  // insert highscores in a gridlayout
  QGridLayout *table = new QGridLayout(12, 3, 5);
  tl->addLayout(table, 1);

  // add a separator line
  KSeparator *sep = new KSeparator(dlg);
  table->addMultiCellWidget(sep, 1, 1, 0, 2);

  // add titles
  f = dlg->font();
  f.setBold(TRUE);

  l = new QLabel("Rank", dlg);
//  l = new QLabel(locale->translate("Rank"), dlg);
  l->setFont(f);
  l->setMinimumSize(l->sizeHint());
  table->addWidget(l, 0, 0);

  l = new QLabel("Name", dlg);
//  l = new QLabel(locale->translate("Name"), dlg);
  l->setFont(f);
  l->setMinimumSize(l->sizeHint());
  table->addWidget(l, 0, 1);

//  l = new QLabel(locale->translate("Score"), dlg);
  l = new QLabel("Score", dlg);
  l->setFont(f);
  l->setMinimumSize(l->sizeHint());
  table->addWidget(l, 0, 2);
  
  QString s;
  QLabel *e[10][3];
  unsigned i, j;

  for(i = 0; i < 10; i++) {
    HighScore hs;
    if(i < highscore.size())
      hs = highscore[i];   
    
    // insert rank    
    s.sprintf("%d", i+1);
    e[i][0] = new QLabel(s.data(), dlg);

    // insert name
    if(i < highscore.size())
      e[i][1] = new QLabel(hs.name, dlg);
    else
      e[i][1] = new QLabel("", dlg);

    // insert time
    if(i < highscore.size()) {
      s.sprintf("%i", hs.score);
      e[i][2] = new QLabel(s.data(), dlg);
    } else
      e[i][2] = new QLabel("", dlg);
      e[i][2]->setAlignment(AlignRight);

  }

  f = dlg->font();
  f.setBold(TRUE);
  f.setItalic(TRUE);
  for(i = 0; i < 10; i++)
    for(j = 0; j < 3; j++) {
      e[i][j]->setMinimumHeight(e[i][j]->sizeHint().height());
      if(j == 1)
	e[i][j]->setMinimumWidth(QMAX(e[i][j]->sizeHint().width(), 100));
      else
	e[i][j]->setMinimumWidth(QMAX(e[i][j]->sizeHint().width(), 60));
      if((int)i == focusitem)
	e[i][j]->setFont(f);
      table->addWidget(e[i][j], i+2, j, AlignCenter);	
    }
    
  QPushButton *b = new QPushButton("Close", dlg);
//  QPushButton *b = new QPushButton(locale->translate("Close"), dlg);
//  if(style() == MotifStyle)
//    b->setFixedSize(b->sizeHint().width() + 10,
//		    b->sizeHint().height() + 10);
//  else
  b->setFixedSize(b->sizeHint());

  // connect the "Close"-button to done
  connect(b, SIGNAL(clicked()),
	  dlg, SLOT(accept()));
  b->setDefault(TRUE);
  b->setFocus();

  // make layout
  tl->addSpacing(10);
  tl->addWidget(b);
  tl->activate();
  tl->freeze();

  dlg->exec();
  delete dlg;
}


void HScore::viewHighScore()
{
    showHighscore(-1);
}

int HScore::getMaxScore()
{
    int m=0;
    for(unsigned int i=0; i<highscore.size(); i++)
      if( highscore[i].score > m ) m = highscore[i].score;

    return m;

}
